branch: externals/org
commit 1686b6f3ba1aff8ae38d51a5ad87e9344236a7eb
Author: Ihor Radchenko <yanta...@posteo.net>
Commit: Ihor Radchenko <yanta...@posteo.net>

    org-attach-dir-from-id: Search existing attachments for symlinks
    
    * lisp/org-attach.el (org-attach-dir-from-id): When current buffer
    displays a symlinked file, search for existing attachments in the
    original file dir.
    * etc/ORG-NEWS (=org-attach= now considers symlinked files when
    searching pre-existing attach dirs): Announce the change.
    
    Reported-by: Karthik Chikmagalur <karthikchikmaga...@gmail.com>
    Link: https://orgmode.org/list/87seyydnf7.fsf@localhost
---
 etc/ORG-NEWS       |  8 ++++++++
 lisp/org-attach.el | 39 +++++++++++++++++++++++++++++----------
 2 files changed, 37 insertions(+), 10 deletions(-)

diff --git a/etc/ORG-NEWS b/etc/ORG-NEWS
index ec28f77d8d..a1efc445fa 100644
--- a/etc/ORG-NEWS
+++ b/etc/ORG-NEWS
@@ -60,6 +60,14 @@ all the references are resolved in the generated png.
 # This also includes changes in function behavior from Elisp perspective.
 
 ** Miscellaneous
+*** =org-attach= now considers symlinked files when searching pre-existing 
attach dirs
+
+When Org buffer is opened from a symlink, Org mode looks into the
+original file directory when searching if an attach directory already exists.
+This way, attachments will remain accessible when opening symlinked Org file.
+
+When no attach dir exists, Org mode will still prefer creating it in
+the "default" directory - where the symlink is located.
 
 * Version 9.7
 
diff --git a/lisp/org-attach.el b/lisp/org-attach.el
index fc7f50a08e..7a03d170ee 100644
--- a/lisp/org-attach.el
+++ b/lisp/org-attach.el
@@ -433,22 +433,42 @@ adjust `org-attach-id-to-path-function-list'"
       (make-directory attach-dir t))
     attach-dir))
 
-(defun org-attach-dir-from-id (id  &optional existing)
+(defun org-attach-dir-from-id (id &optional existing)
   "Return a folder path based on `org-attach-id-dir' and ID.
 Try id-to-path functions in `org-attach-id-to-path-function-list'
 ignoring nils.  If EXISTING is non-nil, then return the first path
-found in the filesystem.  Otherwise return the first non-nil value."
+found in the filesystem.  Otherwise return the first non-nil value.
+
+The existing paths are searched in
+1. `org-attach-id-dir';
+2. in \"data/\" dir - the default value of `org-attach-id-dir';
+3. if current buffer is a symlink, (1) and (2) searches are repeated
+   in the `default-directory' of symlink target."
   (let ((fun-list org-attach-id-to-path-function-list)
         (base-dir (expand-file-name org-attach-id-dir))
-        (default-base-dir (expand-file-name "data/"))
+        (fallback-dirs (list (expand-file-name "data/")))
         preferred first)
+    (when (and (buffer-file-name)
+               (file-symlink-p (buffer-file-name)))
+      (let ((default-directory
+             (file-name-directory
+              (file-truename (buffer-file-name)))))
+        (cl-pushnew (expand-file-name org-attach-id-dir) fallback-dirs)
+        (cl-pushnew (expand-file-name "data/") fallback-dirs)))
+    (setq fallback-dirs (delete base-dir fallback-dirs))
+    (setq fallback-dirs (seq-filter #'file-directory-p fallback-dirs))
     (while (and fun-list
                 (not preferred))
       (let* ((name (funcall (car fun-list) id))
              (candidate (and name (expand-file-name name base-dir)))
-             ;; Try the default value `org-attach-id-dir' as a fallback.
-             (candidate2 (and name (not (equal base-dir default-base-dir))
-                              (expand-file-name name default-base-dir))))
+             ;; Try the default value `org-attach-id-dir', and linked
+             ;; dirs if buffer is a symlink as a fallback.
+             (fallback-candidates
+              (and name (mapcar
+                         (lambda (dir) (expand-file-name name dir))
+                         fallback-dirs)))
+             (fallback-candidates
+              (seq-filter #'file-directory-p fallback-candidates)))
         (setq fun-list (cdr fun-list))
         (when candidate
           (if (or (not existing) (file-directory-p candidate))
@@ -456,10 +476,9 @@ found in the filesystem.  Otherwise return the first 
non-nil value."
             (unless first
               (setq first candidate)))
           (when (and existing
-                     candidate2
-                     (not (file-directory-p candidate))
-                     (file-directory-p candidate2))
-            (setq preferred candidate2)))))
+                     fallback-candidates
+                     (not (file-directory-p candidate)))
+            (setq preferred (car fallback-candidates))))))
     (or preferred first)))
 
 (defun org-attach-check-absolute-path (dir)

Reply via email to